/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes.ejb.seguranca;

import java.security.Principal;
import java.security.acl.Group;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.sql.DataSource;

import org.jboss.security.SimpleGroup;
import org.jboss.security.auth.spi.UsernamePasswordLoginModule;

import br.urca.seguranca.codificacao.UtilsSenha;
import br.urca.solicitacoes.Funcionario;

public class ModuloLogin extends UsernamePasswordLoginModule
{
	private String fHashSalt;
	private String fSituacao;
	
	@Override
	protected String getUsersPassword() throws LoginException
	{
		if (log.isTraceEnabled())
		{
			log.trace("Executando query da senha do usurio '" +
				getUsername() + "'.");
		}
		String sql =
			"SELECT SENHA_2, COMPLEMENTO_SENHA, SITUACAO " +
			"FROM GE_FUNCIONARIOS " +
			"WHERE MATRICULA = ?";
		Dados dados = new Dados(sql, getUsername());
		try
		{
			ResultSet funcionario = dados.getDados();
			if (!funcionario.next())
			{
				if (log.isTraceEnabled())
				{
					log.trace("Usurio '" + getUsername() + 
						"' no encontrado ao executar a query da senha do " +
						"usurio.");
				}
				throw new FailedLoginException(
					"Matrcula e/ou senha invlida.");				
			}
			String senhaCodificada = funcionario.getString(1);
			fHashSalt = funcionario.getString(2);
			fSituacao = funcionario.getString(3);
			return senhaCodificada;
		}
		catch (SQLException ex)
		{
			LoginException excecao = new LoginException("Erro de SQL.");
			excecao.initCause(ex);
			throw excecao;
		}
		finally
		{
			dados.fechar();
		}	
	}
	
	@Override
	protected boolean validatePassword(
		String inputPassword, String expectedPassword)
	{
		if (inputPassword != null && expectedPassword != null)
		{
			String senhaCodificada =
				UtilsSenha.codificarSenha(inputPassword, fHashSalt);
			return senhaCodificada.equals(expectedPassword);
		}
		return false;
	}
	
	@Override
	protected Group[] getRoleSets() throws LoginException
	{
		Group[] grupos = {new SimpleGroup("Roles")};
		Dados dados = null;
		try
		{
			if (Funcionario.ativo(fSituacao))
			{
				if (log.isTraceEnabled())
				{
					log.trace("Executando query de funes do usurio '" + 
						getUsername() + "'.");
				}
				final String FUNCAO_USUARIO = "USUARIO";
				String sql =
					"SELECT COD_FUNCAO " +
					"FROM GE_FUNCOES_FUNCIONARIOS " +
					"WHERE MATRICULA = ?";
				boolean temFuncaoUsuario = false;
				dados = new Dados(sql, getUsername());
				ResultSet funcoes = dados.getDados();
				while (funcoes.next())
				{
					String funcao = funcoes.getString(1);
					adicionarFuncaoAoGrupo(grupos[0], funcao);
					if (funcao.equals(FUNCAO_USUARIO))
					{
						temFuncaoUsuario = true;
					}
				}
				if (!temFuncaoUsuario)
				{
					adicionarFuncaoAoGrupo(grupos[0], FUNCAO_USUARIO);
				}
			}
			else
			{
				adicionarFuncaoAoGrupo(grupos[0], "NAO_AUTORIZADO");
			}
		}
		catch (SQLException ex)
		{
			LoginException excecao = new LoginException("Erro de SQL.");
			excecao.initCause(ex);
			throw excecao;
		}
		catch (Exception ex)
		{
			LoginException excecao = 
				new LoginException("Erro ao criar funo do usurio.");
			excecao.initCause(ex);
			throw excecao;
		}
		finally
		{
			if (dados != null)
			{
				dados.fechar();
			}
		}
		return grupos;
	}
	
	@Override
	protected Throwable getValidateError()
	{
		return new FailedLoginException("Matrcula e/ou senha invlida.");
	}
	
	private void adicionarFuncaoAoGrupo(Group grupo, String nomeFuncao)
		throws Exception
	{
		Principal funcao = createIdentity(nomeFuncao);
		if (log.isTraceEnabled())
		{
			log.trace("Usurio atribudo  funo '" + nomeFuncao + "'.");
		}
		grupo.addMember(funcao);
	}
	
	private static class Dados
	{
		private final Connection fConexao;
		private final PreparedStatement fInstrucao;
		private final ResultSet fDados;

		private Dados(String sql, String matricula) throws LoginException
		{
			try
			{
				InitialContext contexto = new InitialContext();
				DataSource fonteDados =
					(DataSource) contexto.lookup("java:/SolicitacoesDS");
				fConexao = fonteDados.getConnection();
				fInstrucao = fConexao.prepareStatement(sql);
				fInstrucao.setString(1, matricula);
				fDados = fInstrucao.executeQuery();
			}
			catch (NamingException ex)
			{
				LoginException excecao =
					new LoginException("Erro ao localizar fonte de dados.");
		        excecao.initCause(ex);
		        throw excecao;
			}
			catch (SQLException ex)
			{
				fechar();
				LoginException excecao = new LoginException("Erro de SQL.");
				excecao.initCause(ex);
				throw excecao;
			}
		}

		private ResultSet getDados()
		{
			return fDados;
		}

		private void fechar()
		{
			if (fDados != null)
			{
				try
				{
					fDados.close();
				}
				catch (SQLException ex)
				{	            	
				}
			}
			if (fInstrucao != null)
			{
				try
				{
					fInstrucao.close();
				}
				catch (SQLException ex)
				{	            	
				}
			}
			if (fConexao != null)
			{
				try
				{
					fConexao.close();
				}
				catch (SQLException ex)
				{	            	
				}
			}
		}
	}
}