/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes;

public class DadosAdministrador extends DadosFuncionario
	implements Comparable<DadosAdministrador>
{
	private final Funcionario fAdministrador;
	private final Setor fSetorAdministracao;
	private final boolean fFuncionarioAutenticado;
	
	public DadosAdministrador(
		AdministracaoFuncionario administracao, boolean funcionarioAutenticado)
	{
		this(administracao.getFuncionario(), administracao.getSetor(),
			funcionarioAutenticado);
	}
	
	public DadosAdministrador(Funcionario administrador,
		Setor setorAdministracao, boolean funcionarioAutenticado)
	{
		super(
			administrador.getMatricula(),
			administrador.getCodigo(),
			administrador.getNome(),
			administrador.getSetor() == null ?
			null : administrador.getSetor().getNome());
		if (setorAdministracao == null)
		{
			throw new NullPointerException("Setor de administrao nulo.");
		}
		fAdministrador = administrador;
		fSetorAdministracao = setorAdministracao;
		fFuncionarioAutenticado = funcionarioAutenticado;
	}

	public Funcionario getAdministrador()
	{
		return fAdministrador;
	}

	public CategoriaFuncionario getCategoria()
	{
		return fAdministrador.getCategoria();
	}

	public String getSituacao()
	{
		return fAdministrador.getDescricaoSituacao();
	}

	public Setor getSetorAdministracao()
	{
		return fSetorAdministracao;
	}

	public boolean isFuncionarioAutenticado()
	{
		return fFuncionarioAutenticado;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
		{
			return true;
		}
		if (!super.equals(obj))
		{
			return false;
		}
		if (getClass() != obj.getClass())
		{
			return false;
		}
		
		DadosAdministrador outro = (DadosAdministrador) obj;
		
		return fSetorAdministracao.equals(outro.fSetorAdministracao);
	}

	@Override
	public int hashCode()
	{
		final int PRIMO = 31;
		return PRIMO * super.hashCode() + fSetorAdministracao.hashCode();
	}

	public int compareTo(DadosAdministrador o)
	{
		// Nome
		int comparacao = getNome().compareTo(o.getNome());
		if (comparacao != 0)
		{
			return comparacao;
		}
		
		// Setor
		if (getSetor() == null)
		{
			comparacao = o.getSetor() == null ? 0 : -1;
		}
		else
		{
			comparacao =
				o.getSetor() == null ? 1 : getSetor().compareTo(o.getSetor());
		}
		if (comparacao != 0)
		{
			return comparacao;
		}
		
		// Setor de administrao
		comparacao = fSetorAdministracao.getNome().compareTo(
			o.fSetorAdministracao.getNome());
		return comparacao;
	}
}
