/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinColumns;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.Version;

@Entity
@Table(name = "GE_FUNCIONARIOS")
public class Funcionario
{
	private static final String SITUACAO_ATIVO            = "A";
	private static final String SITUACAO_CEDIDO           = "C";
	private static final String SITUACAO_A_DISPOSICAO     = "D";
	private static final String SITUACAO_AFASTADO         = "T";
	private static final String SITUACAO_SUSPENSO_VINCULO = "S";
	private static final String SITUACAO_EXONERADO        = "E";
	private static final String SITUACAO_APOSENTADO       = "F";
	private static final String SITUACAO_INATIVO          = "I";
	private static final String SITUACAO_FALECIDO         = "G";
	
	public static final Set<String> SITUACOES_ATIVAS =
		Collections.unmodifiableSet(new HashSet<String>(Arrays.asList(
			SITUACAO_ATIVO,
			SITUACAO_CEDIDO,
			SITUACAO_A_DISPOSICAO,
			SITUACAO_AFASTADO
	)));

	public static final String FUNCAO_ADMINISTRADOR_GERAL =
		"ADMINISTRADOR_GERAL";

	private String matricula;
	private String nome;
	private Setor setor;
	private Setor setorSolicitacoes;
	private String email;
	private String categoria;
	private String situacao;
	private int codigo;
	private String senha;
	private String hashSalt;
	private boolean senhaAlterada;
	private int versao;
	private List<AdministracaoFuncionario> administracoes =
		new ArrayList<AdministracaoFuncionario>();
	private List<PermissaoFuncionario> permissoes =
		new ArrayList<PermissaoFuncionario>();
	
	@Id
	public String getMatricula()
	{
		return matricula;
	}
	
	public void setMatricula(String matricula)
	{
		this.matricula = matricula;
	}

	public String getNome()
	{
		return nome;
	}
	
	public void setNome(String nome)
	{
		this.nome = nome;
	}

	@ManyToOne
	@JoinColumns({
		@JoinColumn(name = "COD_INSTITUICAO", 
			referencedColumnName = "COD_INSTITUICAO"),
		@JoinColumn(name = "COD_SETOR", 
			referencedColumnName = "COD_SETOR")
	})
	public Setor getSetor()
	{
		return setor;
	}

	public void setSetor(Setor setor)
	{
		this.setor = setor;
	}

	@ManyToOne
	@JoinColumns({
		@JoinColumn(name = "COD_INSTITUICAO_SOLICITACOES", 
			referencedColumnName = "COD_INSTITUICAO"),
		@JoinColumn(name = "COD_SETOR_SOLICITACOES", 
			referencedColumnName = "COD_SETOR")
	})
	public Setor getSetorSolicitacoes()
	{
		return setorSolicitacoes;
	}

	public void setSetorSolicitacoes(Setor setorSolicitacoes)
	{
		this.setorSolicitacoes = setorSolicitacoes;
	}

	@Transient
	public Setor getSetorPadrao()
	{
		return getSetorSolicitacoes() == null ?
			getSetor() : getSetorSolicitacoes();
	}
	
	@Column(name = "E_MAIL")
	public String getEmail()
	{
		return email;
	}

	public void setEmail(String email)
	{
		this.email = email;
	}
	
	@Transient
	public CategoriaFuncionario getCategoria()
	{
		if ("P".equals(getColunaCategoria()))
		{
			return CategoriaFuncionario.PROFESSOR;
		}
		else if ("F".equals(getColunaCategoria()))
		{
			return CategoriaFuncionario.FUNCIONARIO;
		}
		else
		{
			return null;
		}
	}

	@Column(name = "CATEGORIA")
	String getColunaCategoria()
	{
		return categoria;
	}
	
	void setColunaCategoria(String categoria)
	{
		this.categoria = categoria;
	}
	
	public String getSituacao()
	{
		return situacao;
	}

	public void setSituacao(String situacao)
	{
		this.situacao = situacao;
	}

	@Transient
	public String getDescricaoSituacao()
	{
		if (SITUACAO_ATIVO.equals(getSituacao()))
		{
			return "Ativo";
		}
		else if (SITUACAO_CEDIDO.equals(getSituacao()))
		{
			return "Cedido";
		}
		else if (SITUACAO_A_DISPOSICAO.equals(getSituacao()))
		{
			return " disposio";
		}
		else if (SITUACAO_AFASTADO.equals(getSituacao()))
		{
			return "Afastado";
		}
		else if (SITUACAO_SUSPENSO_VINCULO.equals(getSituacao()))
		{
			return "Suspenso de vnculo";
		}
		else if (SITUACAO_EXONERADO.equals(getSituacao()))
		{
			return "Exonerado";
		}
		else if (SITUACAO_APOSENTADO.equals(getSituacao()))
		{
			return "Aposentado";
		}
		else if (SITUACAO_INATIVO.equals(getSituacao()))
		{
			return "Inativo";
		}
		else if (SITUACAO_FALECIDO.equals(getSituacao()))
		{
			return "Falecido";
		}
		else
		{
			return "(situao invlida: '" + getSituacao() + "')";
		}
	}

	@Column(name = "IDENTIFICADOR")
	public int getCodigo()
	{
		return codigo;
	}

	public void setCodigo(int codigo)
	{
		this.codigo = codigo;
	}

	@Column(name = "SENHA_2")
	public String getSenha()
	{
		return senha;
	}

	public void setSenha(String senha)
	{
		this.senha = senha;
	}

	@Column(name = "COMPLEMENTO_SENHA")
	public String getHashSalt()
	{
		return hashSalt;
	}

	public void setHashSalt(String hashSalt)
	{
		this.hashSalt = hashSalt;
	}

	@Column(name = "SENHA_2_MODIFICADA")
	public boolean isSenhaAlterada()
	{
		return senhaAlterada;
	}

	public void setSenhaAlterada(boolean senhaAlterada)
	{
		this.senhaAlterada = senhaAlterada;
	}

	@Version
	public int getVersao()
	{
		return versao;
	}

	public void setVersao(int versao)
	{
		this.versao = versao;
	}

	@OneToMany(mappedBy = "funcionario", cascade = {CascadeType.ALL},
		orphanRemoval = true)
	public List<AdministracaoFuncionario> getAdministracoes()
	{
		return administracoes;
	}

	public void setAdministracoes(List<AdministracaoFuncionario> administracoes)
	{
		this.administracoes = administracoes;
	}

	@OneToMany(mappedBy = "funcionario", cascade = {CascadeType.ALL},
		orphanRemoval = true)
	public List<PermissaoFuncionario> getPermissoes()
	{
		return permissoes;
	}
	
	public void setPermissoes(List<PermissaoFuncionario> permissoes)
	{
		this.permissoes = permissoes;
	}
	
	public boolean administrador(Setor setor)
	{
		for (AdministracaoFuncionario administracao : getAdministracoes())
		{
			if (administracao.getSetor().equals(setor))
			{
				return true;
			}
		}
		return false;
	}

	public Set<TipoPermissao> obterPermissoes(Setor setor)
	{
		Set<TipoPermissao> permissoes;
		if (administrador(setor))
		{
			permissoes = EnumSet.allOf(TipoPermissao.class);
		}
		else
		{
			permissoes = new HashSet<TipoPermissao>();
			for (PermissaoFuncionario permissao : getPermissoes())
			{
				if (permissao.getSetor().equals(setor))
				{
					permissoes.add(permissao.getCodigo().getTipo());
				}
			}
		}
		return permissoes;
	}

	public boolean possuiPermissoes(Setor setor)
	{
		return !obterPermissoes(setor).isEmpty();
	}
	
	public boolean ativo()
	{
		return ativo(getSituacao());
	}

	public static boolean ativo(String situacao)
	{
		return SITUACOES_ATIVAS.contains(situacao);
	}
}