/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes;

import java.util.HashSet;
import java.util.Set;

public final class PermissoesFuncionarioModificaveis
	extends PermissoesFuncionario
{
	private final Set<TipoPermissao> fPermissoes;
	
	public PermissoesFuncionarioModificaveis(
		Funcionario funcionario, Setor setorAtual)
	{
		fPermissoes = obterPermissoes(funcionario, setorAtual);
		inicializar(fPermissoes);
	}
	
	public void setConsultarSolicitacoesPessoais(
		boolean consultarSolicitacoesPessoais)
	{
		super.setConsultarSolicitacoesPessoais(consultarSolicitacoesPessoais);
		atualizarPermissoes(TipoPermissao.CONSULTAR_SOLICITACOES_PESSOAIS,
			consultarSolicitacoesPessoais);
	}

	public void setConsultarTodasSolicitacoes(boolean consultarTodasSolicitacoes)
	{
		super.setConsultarTodasSolicitacoes(consultarTodasSolicitacoes);
		atualizarPermissoes(TipoPermissao.CONSULTAR_TODAS_SOLICITACOES,
			consultarTodasSolicitacoes);
	}
	
	public void setCriarSolicitacoes(boolean criarSolicitacoes)
	{
		super.setCriarSolicitacoes(criarSolicitacoes);
		atualizarPermissoes(TipoPermissao.CRIAR_SOLICITACOES, criarSolicitacoes);
	}
	
	public void setDarAndamentoSolicitacoes(boolean darAndamentoSolicitacoes)
	{
		super.setDarAndamentoSolicitacoes(darAndamentoSolicitacoes);
		atualizarPermissoes(
			TipoPermissao.DAR_ANDAMENTO_SOLICITACOES, darAndamentoSolicitacoes);
	}

	public Set<TipoPermissao> getPermissoes()
	{
		return fPermissoes;
	}
	
	private Set<TipoPermissao> obterPermissoes(
		Funcionario funcionario, Setor setorAtual)
	{
		Set<TipoPermissao> permissoes = new HashSet<TipoPermissao>();
		for (PermissaoFuncionario permissao : funcionario.getPermissoes())
		{
			if (permissao.getSetor().equals(setorAtual))
			{
				permissoes.add(permissao.getCodigo().getTipo());
			}
		}
		return permissoes;
	}
	
	private void atualizarPermissoes(
		TipoPermissao permissao, boolean permissaoConcedida)
	{
		if (permissaoConcedida)
		{
			fPermissoes.add(permissao);
		}
		else
		{
			fPermissoes.remove(permissao);
		}
	}
}
