/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes.web;

import java.io.IOException;
import java.util.EnumSet;

import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.servlet.http.HttpServletRequest;

import br.urca.solicitacoes.DadosFuncionarioAutenticado;

class ControladorAcessoPagina
{
	private static final String DADOS_PESSOAIS = "dadospessoais.xhtml";
	
	void controlarAcessoPagina(PhaseEvent e)
	{
		if (FuncionarioAutenticado.isAutenticado())
		{
			String paginaAtual =
				e.getFacesContext().getViewRoot().getViewId().substring(1);
			if (FuncionarioAutenticado.getInstancia().getSetor() == null)
			{
				exibirPaginaSetorNaoCadastrado(paginaAtual);
			}
			else if (!FuncionarioAutenticado.getInstancia().isSenhaAlterada())
			{
				exibirPaginaAlteracaoSenhaProvisoria(paginaAtual);
			}
			else
			{
				controlarAcessoPaginaAtual(paginaAtual);
			}
		}
	}
	
	private Object obterControladorPagina(String nomeControladorPagina)
	{
		FacesContext contexto = FacesContext.getCurrentInstance();
		Object controladorPagina = contexto.getELContext().getELResolver().
			getValue(contexto.getELContext(), null, nomeControladorPagina);
		return controladorPagina;
	}

	private void exibirPaginaSetorNaoCadastrado(String paginaAtual)
	{
		String mensagem =
			"<span style=\"color: red;\">" +
			"O nosso cadastro no contm a informao de qual setor ou " +
			"departamento voc trabalha.<br />" +
			"Esta informao  necessria para o uso deste sistema.<br />" +
			"Entre em contato com o Departamento Pessoal para atualizar os " +
			"seus dados cadastrais, informando o seu local de trabalho.<br />" +
			"Aps este dado ter sido informado o sistema estar disponvel " +
			"para uso." +
			"</span>";
		Mensagem.definirMensagem(mensagem);
		Mensagem.ocultarBotoes(EnumSet.of(
			Mensagem.Botao.NOVA_SOLICITACAO,
			Mensagem.Botao.CONSULTAR_SOLICITACOES
		));
		
		if (!paginaAtual.equals("setornaocadastrado.xhtml"))
		{
			redirecionarParaPagina("setornaocadastrado.xhtml");
		}
	}

	private void exibirPaginaAlteracaoSenhaProvisoria(String paginaAtual)
	{
		if (!paginaAtual.equals("alteracaosenhaprovisoria.xhtml"))
		{
			redirecionarParaPagina("alteracaosenhaprovisoria.xhtml");
		}
	}

	private void controlarAcessoPaginaAtual(String paginaAtual)
	{
		String pagina = paginaAtual;

		boolean paginaSemRestricoes =
			pagina.equals(DADOS_PESSOAIS) || pagina.equals("mensagem.xhtml");
		if (paginaSemRestricoes)
		{
			return;
		}
		
		DadosFuncionarioAutenticado funcionario =
			FuncionarioAutenticado.getInstancia();
		
		if (funcionario.getPermissoes().existem())
		{
			if (pagina.equals("alteracaosenhaprovisoria.xhtml"))
			{
				pagina = "novasolicitacao.xhtml";
			}
			else if (pagina.equals("gerenciadorfuncionarios.xhtml") &&
				(!funcionario.isAdministradorGeral() &&
				!funcionario.isAdministrador()))
			{
				pagina = "novasolicitacao.xhtml";
			}
			
			if (!funcionario.getPermissoes().isCriarSolicitacoes())
			{
				if (pagina.equals("novasolicitacao.xhtml") ||
					pagina.equals("solicitacaogeral.xhtml") ||
					pagina.equals("confirmacaosolicitacaogeral.xhtml") ||
					pagina.equals("solicitacaomaterial.xhtml") ||
					pagina.equals("confirmacaosolicitacaomaterial.xhtml"))
				{
					pagina = "consultasolicitacoes.xhtml";
				}
			}
			
			if (pagina.equals("consultasolicitacoes.xhtml"))
			{
				if (!funcionario.getPermissoes().
					isConsultarSolicitacoesPessoais() &&
					!funcionario.getPermissoes().isConsultarTodasSolicitacoes())
				{
					pagina = getPaginaAcessivel(funcionario);
				}
			}
			else if (pagina.startsWith("andamentosolicitacao") ||
				pagina.startsWith("confirmacaoandamentosolicitacao"))
			{
				if (!funcionario.getPermissoes().isDarAndamentoSolicitacoes())
				{
					pagina = getPaginaAcessivel(funcionario);
				}
			}
		}
		else
		{
			if (funcionario.isAdministradorGeral())
			{
				if (!pagina.equals("gerenciadorfuncionarios.xhtml"))
				{
					pagina = DADOS_PESSOAIS;
				}
			}
			else
			{
				if (funcionario.getSetores().size() > 1)
				{
					pagina = DADOS_PESSOAIS;
				}
				else
				{
					pagina = "sempermissao.xhtml";
				}
			}
		}
		
		if (!pagina.equals(paginaAtual))
		{
			if (pagina.equals("consultasolicitacoes.xhtml"))
			{
				prepararConsultaSolicitacoes();
			}
			if (pagina.equals(DADOS_PESSOAIS))
			{
				prepararDadosPessoais();
			}
			
			redirecionarParaPagina(pagina);
		}
	}
	
	private void prepararConsultaSolicitacoes()
	{
		ConsultaSolicitacoes consultaSolicitacoes =
			(ConsultaSolicitacoes) obterControladorPagina(
			"consultaSolicitacoes");
		consultaSolicitacoes.exibir();
	}

	private void prepararDadosPessoais()
	{
		DadosPessoais dadosPessoais =
			(DadosPessoais) obterControladorPagina("dadosPessoais");
		dadosPessoais.exibir();
	}

	private String getPaginaAcessivel(DadosFuncionarioAutenticado funcionario)
	{
		if (funcionario.getPermissoes().isCriarSolicitacoes())
		{
			return "novasolicitacao.xhtml";
		}
		else
		{
			return DADOS_PESSOAIS;
		}
	}

	private void redirecionarParaPagina(String pagina)
	{
		try
		{
			if (!pagina.endsWith(".xhtml"))
			{
				pagina = pagina.substring(0, pagina.indexOf(".")) + ".xhtml";
			}
			ExternalContext contextoExterno =
				FacesContext.getCurrentInstance().getExternalContext();
			String caminho =
				((HttpServletRequest) contextoExterno.getRequest()).
				getContextPath();
			pagina = caminho + "/" + pagina;
			contextoExterno.redirect(pagina);
		}
		catch (IOException ex)
		{
			throw new FacesException(ex);
		}	
	}
}
