/**
 * @autor Marcos Antonio Pereira Silva
 */

package br.urca.solicitacoes.web;

import java.util.Arrays;
import java.util.EnumSet;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.FacesContext;

import br.urca.util.UtilsString;

@ManagedBean
@SessionScoped
public class Mensagem
{
	private static final String CHAVE_MENSAGEM =
		"br.urca.solicitacoes.web.MENSAGEM";
	private static final String CHAVE_BOTOES_VISIVEIS =
		"br.urca.solicitacoes.web.BOTOES_VISIVEIS";
	
	public String getMensagem()
	{
		String mensagem =
			(String) getParametrosRequisicao().get(CHAVE_MENSAGEM);
		return mensagem == null ? "" : mensagem;
	}
	
	public void setMensagem(String mensagem)
	{
		definirMensagem(mensagem);
	}
	
	public boolean isBotaoNovaSolicitacaoVisivel()
	{
		if (FuncionarioAutenticado.getInstancia().getPermissoes().
			isCriarSolicitacoes())
		{
			return getBotoesVisiveis().contains(Botao.NOVA_SOLICITACAO);
		}
		else
		{
			return false;
		}
	}
	
	public boolean isBotaoConsultarSolicitacoesVisivel()
	{
		if (FuncionarioAutenticado.getInstancia().getPermissoes().
			isConsultarSolicitacoesPessoais() ||
			FuncionarioAutenticado.getInstancia().getPermissoes().
			isConsultarTodasSolicitacoes())
		{
			return getBotoesVisiveis().contains(Botao.CONSULTAR_SOLICITACOES);
		}
		else
		{
			return false;
		}
	}	

	public boolean isBotaoSairVisivel()
	{
		return getBotoesVisiveis().contains(Botao.SAIR);
	}
	
	public int getTotalLinhas()
	{
		return UtilsString.contarOcorrenciasExpressaoRegular(
			"<(b|B)(r|R)\\s*/>", getMensagem()) + 1;
	}
	
	static void definirMensagem(String mensagem)
	{
		getParametrosRequisicao().put(CHAVE_MENSAGEM, mensagem);
	}
	
	static void ocultarBotoes(Set<Botao> botoes)
	{
		Set<Botao> botoesVisiveis =
			new HashSet<Botao>(Arrays.asList(Botao.values()));
		botoesVisiveis.removeAll(botoes);
		setBotoesVisiveis(botoesVisiveis);
	}
	
	static void ocultarBotoes(Botao botao)
	{
		ocultarBotoes(EnumSet.of(botao));
	}
	
	private Set<Botao> getBotoesVisiveis()
	{
		@SuppressWarnings("unchecked")
		Set<Botao> botoesVisiveis =
			(Set<Botao>) getParametrosRequisicao().get(CHAVE_BOTOES_VISIVEIS);
		if (botoesVisiveis == null || botoesVisiveis.isEmpty())
		{
			botoesVisiveis = new HashSet<Botao>(Arrays.asList(Botao.values()));
		}
		return botoesVisiveis;
	}
	
	private static void setBotoesVisiveis(Set<Botao> botoesVisiveis)
	{
		getParametrosRequisicao().put(CHAVE_BOTOES_VISIVEIS, botoesVisiveis);
	}
	
	private static Map<String, Object> getParametrosRequisicao()
	{
		return FacesContext.getCurrentInstance().getExternalContext().
			getRequestMap();
	}
	
	enum Botao
	{
		NOVA_SOLICITACAO,
		CONSULTAR_SOLICITACOES,
		SAIR
	}
}